% a_newton.m
% This script solves Power Flow equations using Newton-Raphshon method. The
% maximum number of iterations and maximum tolerance are defined in init.m
%
% Needs:   buQmin, 
%          buQmax: Generator Q limits
%              pv,
%              pq: indices of PV and PQ buses
%              Vm,
%              Va: initial guess of bus voltage magnitude and angles defined
%                  in a_precalc.m or taken from file.
%               V: complex voltage
% Defines:     Vm, 
%              Va: power flow solutions
%          calcQg, 
%          calcPg: calculated generator active and reactive power
%
% Part of the code written for GPFM strategy,
% submitted to IEEE Transactions on Smart Grid, July 2011.
%
% Copyright (c) 2012 Ali Mehrizi-Sani
% Washington State University
% mehrizi@eecs.wsu.edu
% http://eecs.wsu.edu/~mehrizi
% (509) 335-6249


% Set up indexing for updating V and forming Jacobian matrix
a_setindices;

% Evaluate delPQ(x0)
Sbus = (buPg + j*buQg - buPd - j*buQd)/baseMVA;
mis = V .* conj(Ybus * V) - Sbus;
delPQ = [  real(mis(pv));           % dP for PV buses
           real(mis(pq)) ;          % dP for PQ buses
           imag(mis(pq)) ];         % dQ for PQ buses

disp(sprintf('\nIteration    Error'));
disp(        '----------------------');

% Initialize loop variables
iter = 0;
converged = 0;

while (~converged & iter < maxiter)
    iter = iter + 1;
    
    % Calculate Jacobian
    
    % Define helping variables
    Ibus = Ybus * V;
    diagV     = diag(V);
    diagIbus  = diag(Ibus);
    diagVnorm = diag(V./abs(V));

    % Find derivative of S (=P + jQ) w.r.t. |V| and /_V
    dSbus_dVmag = diagV * conj(Ybus * diagVnorm) + conj(diagIbus) * diagVnorm;
    dSbus_dVang = j * diagV * conj(diagIbus - Ybus * diagV);

    % Separate real and imaginary parts and use the nonzero derivatives...
    dP_dVang = real(dSbus_dVang([pv; pq], [pv; pq]));
    dP_dVmag = real(dSbus_dVmag([pv; pq], pq));
    dQ_dVang = imag(dSbus_dVang(pq ,[pv; pq]));
    dQ_dVmag = imag(dSbus_dVmag(pq, pq));
    
    % ...and make Jacobian
    J = [dP_dVang dP_dVmag
         dQ_dVang dQ_dVmag];
   
    % Compute increment to |V| and /_V
    dx = -(J \ delPQ);

    % Update voltage magnitudes and angles
    if npv
        Va(pv) = Va(pv) + dx(j1:j2);
    end
    if npq
        Va(pq) = Va(pq) + dx(j3:j4);
        Vm(pq) = Vm(pq) + dx(j5:j6);
    end
    
    % Update V vector
    V = Vm .* exp(j * Va);
    % Correct Vm and Va in case Vm is negative, just in case
    Vm = abs(V);
    Va = angle(V);

    % Evalute delPQ(x)
    mis = V .* conj(Ybus * V) - Sbus;
	delPQ = [  real(mis(pv));           % dP for PV buses
               real(mis(pq)) ;          % dP for PQ buses
               imag(mis(pq)) ];         % dQ for PQ buses
        
    % Calculate error
    err = norm(delPQ, inf);
    
    % Show the interation number and error
    disp(sprintf('%3d        %10.3e', iter, err));

    % Check error
    if err < tol
        converged = 1;
        disp(sprintf(['\nPower Flow solution has converged in %d '...
                'iterations.'], iter));
    end
    
    % If in early stages, it is too early to decide about Q limits
    % NOTE: this causes the program to iterate at least three times
    if iter < 3, continue, end

    % Otherwise, enforce reactive power limits
    % Calculate Qg for each gen bus
    calcQg = imag(mis + Sbus) * baseMVA + buQd;

    % ...and compare with limits
    ExceedQmax = find(calcQg > buQmax & buType == PV_BUS);
    ExceedQmin = find(calcQg < buQmin & buType == PV_BUS);
    ViolateQLimit = [ExceedQmax ExceedQmin];

    for k =1:length(ViolateQLimit)
        i = ViolateQLimit(k);
        disp(sprintf('\nBus number %d has exceeded its ', i));
        
        % Glue the generator Q to the limit...
        if find(i==ExceedQmax)
            buQg(i) = buQmax(i);
            disp(sprintf('Qmax = %f', buQmax(i)));            
        else
            buQg(i) = buQmin(i);
            disp(sprintf('Qmin = %f', buQmin(i)));                        
        end
        % ... and change bus type
        buType(i) = PQ_BUS;

        % Update indices
        a_setindices;

        % Recalculate power mismatches
        Sbus = (buPg + j*buQg - buPd - j*buQd)/baseMVA;
        mis = V .* conj(Ybus * V) - Sbus;
		delPQ = [  real(mis(pv));           % dP for PV buses
                   real(mis(pq)) ;          % dP for PQ buses
                   imag(mis(pq)) ];         % dQ for PQ buses
    end
           
% originally test of err < tol was here
    
end

% Calculate final soultions
Vm     = Vm; 
Va_deg = Va * 180/pi;
calcQg = imag(mis + Sbus) * baseMVA + buQd;
calcPg = real(mis + Sbus) * baseMVA + buPd;