% at_DStates.m
% This script calculates time derivatives of all of system states. If more
% components are to be modeled or more modeled are to be used (e.g.
% induction motor model), it should be added here in network interface
% part. Also, for simulation of a manual exciter or third-order exciter,
% the lines indicated in the code should be commented out.
% For solution of algebraic equations, Gauss-Seidel method is used.
%
% Needs: 
%               xx: Current states of the system
%             nGen: Number of generators
%             nBus: Number of buses
%
% Defines:
%               px: Vector of the system states derivates
%
% Part of the code written for GPFM strategy,
% submitted to IEEE Transactions on Smart Grid, July 2011.
%
% Copyright (c) 2012 by Ali Mehrizi-Sani
% Washington State University
% mehrizi@eecs.wsu.edu
% http://eecs.wsu.edu/~mehrizi
% (509) 335-6249


% Define aliases for states
dw     = xx( 0*nGen + [1:nGen]);
delta  = xx( 1*nGen + [1:nGen]);
psi_fd = xx( 2*nGen + [1:nGen]);
psi_1d = xx( 3*nGen + [1:nGen]);
psi_1q = xx( 4*nGen + [1:nGen]);
psi_2q = xx( 5*nGen + [1:nGen]);
v_1    = xx( 6*nGen + [1:nGen]);
v_2    = xx( 7*nGen + [1:nGen]);
v_3    = xx( 8*nGen + [1:nGen]);
v_4    = xx( 9*nGen + [1:nGen]);
E_fd   = xx(10*nGen + [1:4]);                   % comment this to simulate a manual exciter
x_pER  = xx(11*nGen +        [1:nBus]);
x_qER  = xx(11*nGen + nBus + [1:nBus]);

% Calculate subtransient dq voltages
Ed_d = -Xd_aqs .* (psi_1q./X_1q + psi_2q./X_2q);
Ed_q =  Xd_ads .* (psi_fd./X_fd + psi_1d./X_1d);

% Find RI-frame Ed
ss = sin(delta); cc = cos(delta);
Ed =  ( Ed_d.*ss + Ed_q.*cc) +...
    j*(-Ed_d.*cc + Ed_q.*ss);

% INTERFACE WITH THE NETWORK. Refer to the report for more information.
% Define internal voltages vector
Ed_vector = [zeros(nLoad, 1); Ed];

% Initialize solver variables
converged = false;
gsiter = 0; gsMaxIter = 100;
tol = 1e-5;

% Solve network and device algebraic equations
while (~converged && gsiter < gsMaxIter)
    gsiter = gsiter + 1;

    % Model IP (not ZIP!) loads
    V_busm = abs(V_bus);
    P_dZIP = P_d0 .* (K_pi.*V_busm./V_busm0 + K_pp.*(V_busm./V_busm0).^2);
    Q_dZIP = Q_d0 .* (K_qi.*V_busm./V_busm0 + K_qq.*(V_busm./V_busm0).^2);
    
    % Model ER loads
    P_tER = P_d0 .* K_pe .* (V_busm./V_busm0).^n_ptER;
    Q_tER = Q_d0 .* K_qe .* (V_busm./V_busm0).^n_qtER;
    
    P_dER = x_pER + P_tER;
    Q_dER = x_qER + Q_tER;
    
    % Calculate injected current from loads (negative)
    I_inj_load = - ((P_dZIP+P_dER) - j*(Q_dZIP+Q_dER))./conj(V_bus);

    % Find value of voltages (Guass-Seidel method)
    V_bus_new = Z_gbus * (I_inj_load + Yg_matrix * Ed_vector);
    
    % Converged?
    if max(abs(V_bus_new - V_bus)) < tol
        converged = true;
    end
    
    % Update bus voltages vector
    V_bus = V_bus_new;
end

% FIND TIME DERIVATIVE
% Find injected currents of the generators
I_inj = Y_bus * V_bus;
I_tgen = I_inj(genBus);

% Find dq-axis currents
i_d = real(I_tgen).*ss - imag(I_tgen).*cc;
i_q = real(I_tgen).*cc + imag(I_tgen).*ss;

% Find aux fluxes psi_ad and psi_aq
psi_ad = Xd_ads .* (-i_d + psi_fd./X_fd + psi_1d./X_1d);
psi_aq = Xd_aqs .* (-i_q + psi_1q./X_1q + psi_2q./X_2q);

% Calculate electrical torque (R_a = 0)
T_e = psi_ad.*i_q - psi_aq.*i_d;

% Calculate field voltage
e_fd = R_fd./X_adu .* E_fd;

% Calculate time derivates of generator states
pDW     = 1./(2*H) .* (T_m - T_e - D.*dw);
pDelta  = w_0 * dw;
pPsi_fd = w_0 * (e_fd + (psi_ad - psi_fd)./X_fd.*R_fd);
pPsi_1d = w_0 * (       (psi_ad - psi_1d)./X_1d.*R_1d);  
pPsi_1q = w_0 * (       (psi_aq - psi_1q)./X_1q.*R_1q);
pPsi_2q = w_0 * (       (psi_aq - psi_2q)./X_2q.*R_2q);

% Calculate time derivatives of PSS states
pV_1 = 1./T_R .* (abs(V_bus(genBus)) - v_1);
pV_2 = K_STAB .* pDW - 1./T_W.*v_2;
pV_3 = 1./T_2 .* (T_1.*pV_2 + v_2 - v_3);

% Calculate time derivatives of exciter states
v_s = max(min(v_3, v_smax), v_smin);        % (non-windup limiter)
v_x = V_ref - v_1 + v_s;
pV_4 = 1./T_B .* (v_x + T_A.*(-pV_1 + pV_3) - v_4);
pE_fd = 1./T_E .* (K_A.*v_4 - E_fd);

% Enforce wind-up limiter for E_fd, my fac part of this code :D
xceedMax = find(E_fd > E_fmax);
posD = find(pE_fd(xceedMax) > zeros(length(xceedMax), 1));
pE_fd(posD) = 0; E_fd(posD) = E_fmax(posD);

xceedMin = find(E_fd < E_fmin);
negD = find(pE_fd(xceedMin) < zeros(length(xceedMin), 1));
pE_fd(negD) = 0; E_fd(negD) = E_fmin(negD);
    
% Uncomment for a third order model.

% v_s = max(min(v_3, v_smax), v_smin);         % wind-up limiter
% 
% E_fd = K_A .* (V_ref - v_1 + v_s);
% E_fd = max(min(E_fd, E_fmax), E_fmin);

% Exponential recovery model parameters
P_sER = P_d0 .* K_pe .* (V_busm./V_busm0).^n_psER;
Q_sER = Q_d0 .* K_qe .* (V_busm./V_busm0).^n_qsER;

pX_pER = (P_sER - P_dER)./T_pER;
pX_qER = (Q_sER - Q_dER)./T_qER;

% If more models are to be used, add here. For example, states of an
% induction motor can be defined here

% Output derivatives
px = [pDW
      pDelta
      pPsi_fd
      pPsi_1d
      pPsi_1q
      pPsi_2q
      pV_1
      pV_2
      pV_3
      pV_4
      pE_fd
      pX_pER
      pX_qER
  ];