% at_DyrRead.m
% This script loads the generator and exciter data from the default dyr
% file. Each generator should be accompanied with an exciter. 'GENROU',
% 'GENSAL', and 'SCRX' are recognized. Stator resistance and sub-transient
% saliency are ignored, as there is no data for them in PSS/E file format.
%
% Needs: 
%          baseMVA: Base MVA for the system
%          geMBase: Base MVA for individual generators
%             nGen: Number of generators
%
% Defines:    H, D: Inertia and damping constants
%         X_d, X_q,
%       X_fd, X_1d,
%       X_1q, X_2q,
%       R_fd, R_1d,
%       R_1q, R_2q: Generator rotor circuit reactances
%            y_gen: Generator admittance
% K_STAB, T_W, T_R,
%         T_1, T_2,
%   v_smax, v_smin: PSS Parameters
%         T_B, T_A,
%         T_E, K_A,
%   E_fmin, E_fmax: Exciter parameters
%
% Part of the code written for GPFM strategy,
% submitted to IEEE Transactions on Smart Grid, July 2011.
%
% Copyright (c) 2012 by Ali Mehrizi-Sani
% Washington State University
% mehrizi@eecs.wsu.edu
% http://eecs.wsu.edu/~mehrizi
% (509) 335-6249


% Initialize generator parameters to a vertical zero vector to prevent "frauds"
H = zeros(nGen, 1);
D = zeros(nGen, 1);
X_d = zeros(nGen, 1);
X_q = zeros(nGen, 1);
Xp_d = zeros(nGen, 1);
Xp_q = zeros(nGen, 1);
Xd_d = zeros(nGen, 1);
X_l = zeros(nGen, 1);
R_a = zeros(nGen, 1);

Tp_d0 = zeros(nGen, 1);
Td_d0 = zeros(nGen, 1);
Tp_q0 = zeros(nGen, 1);
Td_q0 = zeros(nGen, 1);

% Initialize exciter parameters to vertical zero vector to prevent "frauds"
K_STAB = zeros(nGen, 1);
T_W = zeros(nGen, 1);
T_1 = zeros(nGen, 1);
T_2 = zeros(nGen, 1);
v_smax = zeros(nGen, 1);
v_smin = zeros(nGen, 1);
T_R = zeros(nGen, 1);

T_B = zeros(nGen, 1);
T_A = zeros(nGen, 1);
T_E = zeros(nGen, 1);
K_A = zeros(nGen, 1);
E_fmin = zeros(nGen, 1);
E_fmax = zeros(nGen, 1);

% Initiaize counters
Igensal = 0;
Igenrou = 0;
Iscrx = 0;

% Open the default dyr file
fid = fopen('12busAC.dyr');

% Loop to read the file content. In each iteration, all parameters of one
% device is obtained.
while ~feof(fid)
    tline = upper(fgets(fid));
    
	if ~isempty(strfind(tline, 'GENSAL'))
        Igensal = Igensal + 1;
        ind = Igensal + Igenrou;
        
        [gsrBus(ind) gsrType(ind) dummyi Tp_d0(ind) Td_d0(ind) Td_q0(ind) H(ind)] =...
            strread(tline, '%d %s %d %f %f %f %f', 1, 'delimiter', ' ');

        tline = fgets(fid);
        [D(ind) X_d(ind) X_q(ind) Xp_d(ind) Xd_d(ind)] =...
            strread(tline, '%f %f %f %f %f', 1, 'delimiter', ' ');
             
        tline = fgets(fid);
        [X_l(ind) S10 S12 dummy] =...
            strread(tline, '%f %f %f %s', 1, 'delimiter', ' ');
        
        % In order to be able to use the same set of equations for both
        % GENSAL and GELROU, values of Td_q0 and Xd_d are used for Tp_q0
        % and Xp_q. Also, X_2q will be set to infinity and R_2q is set to zero.
        Tp_q0(ind) = Td_q0(ind);
        Xp_q(ind) = Xd_d(ind);
        
	elseif ~isempty(strfind(tline, 'GENROU'))
        Igenrou = Igenrou + 1;
        ind = Igensal + Igenrou;
        
        [gsrBus(ind) gsrType(ind) dummyi Tp_d0(ind) Td_d0(ind) Tp_q0(ind) Td_q0(ind)] =...
            strread(tline, '%d %s %d %f %f %f %f', 1, 'delimiter', ' ');
        
        tline = fgets(fid);
        [H(ind) D(ind) X_d(ind) X_q(ind) Xp_d(ind)] =...
            strread(tline, '%f %f %f %f %f', 1, 'delimiter', ' ');
        
        tline = fgets(fid);
        [Xp_q(ind) Xd_d(ind) X_l(ind) S10 S12 dummy] =...
            strread(tline, '%f %f %f %f %f %s', 1, 'delimiter', ' ');
        
	elseif ~isempty(strfind(tline, 'SCRX'))
        Iscrx = Iscrx + 1;
        ind = Iscrx;
        
        [excBus(ind) excType(ind) dummyi Ta_Tb T_B(ind) K_A(ind) T_E(ind)] =...
            strread(tline, '%d %s %d %f %f %f %f', 1, 'delimiter', ' ');
        
        tline = fgets(fid);
        [E_fmin(ind) E_fmax(ind) CSWITCH rc_rfd dummy] =...
            strread(tline, '%f %f %f %f %s', 1, 'delimiter', ' ');
        
        T_A(ind) = Ta_Tb * T_B(ind);
        
        % Parameters of the PSS are missing, so using default ones
        K_STAB(ind) = 9.5;
        T_W(ind) = 1.41;
        T_1(ind) = 0.154;
        T_2(ind) = 0.033;
        v_smax(ind) = 0.2;
        v_smin(ind) = -0.2;
        T_R(ind) = 0.015;
                  
	else
        disp('\nWARNING: Not recognized command. Exiting the program...')
        return
	end

end

% Close the data file
fclose(fid);

% Calculate base frequency
w_0 = 2*pi*60;

% Find system-base values of reactances and H and D
H = H .* geMBase/baseMVA;
D = D .* geMBase/baseMVA;
X_d = X_d * baseMVA./geMBase;
X_q = X_q * baseMVA./geMBase;
Xp_d = Xp_d * baseMVA./geMBase;
Xp_q = Xp_q * baseMVA./geMBase;
Xd_d = Xd_d * baseMVA./geMBase;
X_l = X_l * baseMVA./geMBase;
R_a = 0 * baseMVA./geMBase;

% Find circuit reactances from d- and q-axis transient and subtransient
% reactances. X_1q can be found from the same equation for both GENSAL and
% GENROU by setting Xp_q = Xd_q as stated before. Also sets X_2q for GENSAL
% to a very high value to remove the effect of psi_2q
Xd_q = Xd_d;
X_ad = X_d - X_l;
X_aq = X_q - X_l;
X_fd = X_ad .* (Xp_d - X_l) ./ (X_ad + X_l - Xp_d);
X_1d = -X_ad.*X_fd.*(-Xd_d+X_l) ./ (-Xd_d.*X_ad - Xd_d.*X_fd + X_l.*X_ad + X_l.*X_fd + X_ad.*X_fd);
X_1q = -X_aq .* (-Xp_q+X_l) ./ (-Xp_q + X_l + X_aq);           
warning off MATLAB:divideByZero
X_2q = -X_aq.*X_1q.*(-Xd_q+X_l) ./ (-Xd_q.*X_aq - Xd_q.*X_1q + X_l.*X_aq + X_l.*X_1q + X_aq.*X_1q);
warning on MATLAB:divideByZero
X_2q(find(strcmp(gsrType', '''GENSAL'''))) = 1e8;

% Find circuit resistances from damper d- and q-axis rectances and field
% circuit reactance, as well as d- and q- axis transient and sub-transient
% time constants. R_1q can be found from the same equation for both GENSAL and
% GENROU by setting Tp_q0 = Td_q0 as stated before. Also sets R_2q for GENSAL
% to zero to ignore the rate of change of pPsi_2q.
R_fd = (X_ad + X_fd) ./ (Tp_d0*w_0);
R_1d = (X_1d + X_ad.*X_fd./(X_ad+X_fd)) ./ (Td_d0*w_0);
R_1q = (X_aq + X_1q) ./ (Tp_q0*w_0);
R_2q = (X_2q + (X_aq.*X_1q) ./ (X_aq+X_1q)) ./ (Td_q0*w_0);
R_2q(find(strcmp(gsrType', '''GENSAL'''))) = 0;
    
% Define saturation constants (not modeled)
K_sd = 1;
K_sq = K_sd;

% Define unsaturated reactances
X_adu = X_ad;
X_aqu = X_aq;

% Define saturated reactances
X_ad = K_sd*X_ad;
X_aq = K_sq*X_aq;
X_d = X_ad + X_l;
X_q = X_aq + X_l;
Xd_ads = Xd_d - X_l;
Xd_aqs = Xd_q - X_l;

% Define generator subtransient admittance neglecting sub-transient
% saliency
y_gen = (R_a + j*Xd_d) .^ -1;

% Clear helping variables
clear Igensal Igenrou Iscrx
clear X_l K_sd K_sq Xp_d Xp_q Xd_d Xd_q
clear Tp_d0 Tp_q0 Td_d0 Td_q0 S10 S20
clear CSWITCH rc_rfd dummy dummyi Ta_Tb
clear tline ind fid